package ui

import (
	"github.com/aditya-K2/tview"
	"github.com/gdamore/tcell/v2"
)

var (
	ImgY int
	ImgW int
	ImgH int
	ImgX int
	Ui   *Application
)

type Application struct {
	App          *tview.Application
	ExpandedView *tview.Table
	Navbar       *tview.Table
	SearchBar    *tview.InputField
	ProgressBar  *ProgressBar
	Pages        *tview.Pages
}

func NewApplication() *Application {

	// progress bar
	progressBar := NewProgressBar()
	progressBar.SetProgressFunc(ProgressFunction)

	// main view
	expandedView := tview.NewTable()
	expandedView.SetBackgroundColor(tcell.ColorDefault)
	expandedView.SetBorderPadding(0, 1, 1, 1).SetBorder(false)
	expandedView.SetSelectable(true, false)

	// image preview
	imagePreviewer := tview.NewBox()
	imagePreviewer.SetBorder(false)
	imagePreviewer.SetDrawFunc(func(
		screen tcell.Screen,
		x, y,
		width, height int,
	) (
		int, int,
		int, int,
	) {
		ImgX, ImgY, ImgW, ImgH = imagePreviewer.GetRect()
		return imagePreviewer.GetInnerRect()
	})
	imagePreviewer.SetBackgroundColor(tcell.ColorDefault)

	// nav bar
	navBar := tview.NewTable()
	navBar.SetBackgroundColor(tcell.ColorDefault)
	navBar.SetBorderPadding(1, 1, 2, 2).SetBorder(false)
	navBar.SetBackgroundColor(tcell.ColorBlack)
	navBar.SetSelectable(true, false)
	navBar.SetSelectedStyle(
		tcell.Style{}.
		Background(tcell.ColorGrey))
	navBar.SetCell(0, 0, tview.NewTableCell("playlist"))
	navBar.SetCell(1, 0, tview.NewTableCell("files"))
	navBar.SetCell(2, 0, tview.NewTableCell("most played"))
	navBar.SetCell(3, 0, tview.NewTableCell("search"))

	// search bar
	searchBar := tview.NewInputField()
	searchBar.SetFieldBackgroundColor(tcell.ColorDefault)
	searchBar.SetBackgroundColor(tcell.ColorDefault)
	searchBar.SetTitle("search").SetTitleAlign(tview.AlignLeft)
	searchBar.SetAutocompleteBackgroundColor(tcell.ColorBlack)
	searchBar.SetAutocompleteSelectBackgroundColor(tcell.ColorGray)
	searchBar.SetAutocompleteMainTextColor(tcell.ColorDefault)
	searchBar.SetAutocompleteSelectedTextColor(tcell.ColorDefault)
	searchBar.SetBorder(true)
	searchBar.SetBorderColor(tcell.ColorGray)
	
	// layout
	sidebar := tview.NewFlex().SetDirection(tview.FlexRow).
		AddItem(navBar, 0, 4, false).
		AddItem(imagePreviewer, 9, 3, false)

	middleRow := tview.NewFlex().
		AddItem(sidebar, 17, 1, false).
		AddItem(expandedView, 0, 4, false)

	mainLayout := tview.NewFlex().SetDirection(tview.FlexRow).
		AddItem(searchBar, 3, 1, false).
		AddItem(middleRow, 0, 1, false).
		AddItem(progressBar, 5, 1, false)

	// pages
	rootPages := tview.NewPages()
	rootPages.AddPage("Main", mainLayout, true, true)

	// application
	App := tview.NewApplication()
	App.SetRoot(rootPages, true).SetFocus(expandedView)

	searchBar.SetDoneFunc(func(k tcell.Key) {
		switch k {
		case tcell.KeyEscape:
			{
				App.SetFocus(expandedView)
			}
		}
	})

	return &Application{
		App:          App,
		ExpandedView: expandedView,
		Navbar:       navBar,
		SearchBar:    searchBar,
		ProgressBar:  progressBar,
		Pages:        rootPages,
	}

}
